<?php

namespace Drupal\sdc\Element;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Render\Element\RenderElement;

/**
 * Provides a Storybook component render element.
 *
 * Properties:
 * - #component: The machine name of the component.
 *
 * Usage Example:
 *
 * @code
 * $build['component'] = [
 *   '#type' => 'sdc',
 *   '#component' => 'button',
 * ];
 * @endcode
 *
 * @see \Drupal\Core\Render\Element\Textarea
 *
 * @RenderElement("sdc")
 */
class ComponentElement extends RenderElement {

  /**
   * Expands a sdc into an inline template with an attachment.
   *
   * @param array $element
   *   The element to process. See main class documentation for properties.
   *
   * @return array
   *   The form element.
   *
   * @throws \Drupal\sdc\Exception\ComponentNotFoundException
   */
  public static function preRenderComponent(array $element): array {
    $bubbleable_metadata = new BubbleableMetadata();
    $context = $element['#context'] ?? [];
    $context_alter_callback = $element['#context_alter_callback'] ?? NULL;
    if (is_callable($context_alter_callback)) {
      $context = $context_alter_callback($context, $bubbleable_metadata);
    }
    $slots_alter_callback = $element['#slots_alter_callback'] ?? NULL;
    $id = $element['#component'];
    $slots = $element['#slots'] ?? [];
    $trusted_slots = $element['#trusted_slots'] ?? TRUE;
    $inline_template = static::generateComponentTemplate(
      $id,
      $slots,
      $slots_alter_callback,
      $bubbleable_metadata,
      $trusted_slots,
    );
    $element['inline-template'] = [
      '#type' => 'inline_template',
      '#template' => $inline_template,
      '#context' => $context,
    ];
    $bubbleable_metadata->applyTo($element['inline-template']);
    return $element;
  }

  /**
   * Generates the template to render the component.
   *
   * @param string $id
   *   The component id.
   * @param array $slots
   *   The contents of any potential embed blocks.
   * @param callable|null $slots_alter_callback
   *   The potential callable for altering slots.
   * @param \Drupal\Core\Cache\CacheableMetadata $bubbleable_metadata
   *   The cacheable metadata.
   * @param bool $trusted_slots
   *   Set to TRUE to render the twig blocks verbatim instead of filtering them.
   *
   * @return string
   *   The template.
   */
  private static function generateComponentTemplate(
    string $id,
    array $slots,
    mixed $slots_alter_callback,
    CacheableMetadata $bubbleable_metadata,
    bool $trusted_slots = FALSE,
  ): string {
    $template = '{# This template was dynamically generated by sdc #}' . PHP_EOL;
    $template .= sprintf('{%% embed \'%s\' %%}', $id);
    $template .= PHP_EOL;
    foreach ($slots as $block_name => $block_value) {
      $bl_val = is_callable($slots_alter_callback)
        ? $slots_alter_callback($block_value, $bubbleable_metadata)
        : $block_value;
      $block_build = $trusted_slots
        ? ['#markup' => Xss::filterAdmin($bl_val)]
        : array_filter([
          '#type' => 'processed_text',
          '#text' => $bl_val,
          '#format' => is_array($block_value) ? $block_value['format'] : NULL,
        ]);
      try {
        $value = \Drupal::service('renderer')->render($block_build);
      }
      catch (\Exception $e) {
        $value = $bl_val;
      }
      // Save the collected attachments before we turn into a string.
      $twig_block_metadata = BubbleableMetadata::createFromRenderArray($block_build);
      $bubbleable_metadata = $bubbleable_metadata->merge($twig_block_metadata);
      $template .= "  {% block $block_name %}" . PHP_EOL
        . "    $value" . PHP_EOL
        . "  {% endblock %}" . PHP_EOL;
    }
    $template .= '{% endembed %}' . PHP_EOL;
    return $template;
  }

  /**
   * {@inheritdoc}
   */
  public function getInfo(): array {
    $class = static::class;
    return [
      '#pre_render' => [
        [$class, 'preRenderComponent'],
      ],
      '#component' => '',
      '#context' => [],
    ];
  }

}
